(*
 *                                                                                             .
 * Module name: nf.dpr; Module type: Delphi (console) project; Language: Object pascal;        .
 * Original developed with: Delphi 5 Enterprise; Portability: most likely down to Delphi 2;    .
 *                                                                                             .
 * Author: ~SoffyT (web:<http://www.nefzger.at/~SoffyT/>; mailto:<MissyQueen@nefzger.at>)      .
 * File created: Fri., 2002-11-01; Last modified: Sat., 2005-10-22                             .
 *                                                                                             .
 * Bug report: Pls. report all bugs, wishes, suggestions etc. to <BugReport@nefzger.at>.       .
 *                                                                                             .
 *  Purpose:                                                                                   .
 * ----------                                                                                  .
 *    Console wrapper for 'Fakultaet.pas'                                                      .
 *                                                                                             .
 *)

(*
 *                                                                                             .
 * To compile with Delphi:                                                                     .
 *    (To do ..)                                                                               .
 * To compile with FPC:                                                                        .
 *    fpc -B -Sd -Op2 -Xs -XX -Fu"..\includes;..\core_utilities;..\bogus_classes" -FU"%TEMP%" nf.dpr -onf.fpc.exe            .
 *                                                                                             .
 *)

{$APPTYPE Console}
program
   nFactorial; // AKA 'nf' or 'nf.dpr'                                                         .

{$INCLUDE '..\includes\Globals.inc'}

uses
   Windows,
   SysUtils,
{$IFDEF DelphiPascalCompiler}
   CmdLine_Utilities in '..\core_utilities\CmdLine_Utilities.pas',
   Misc_Utilities in '..\core_utilities\Misc_Utilities.pas',
   Fakultaet in '..\bogus_classes\Fakultaet.pas';
{$ENDIF}
{$IFDEF FreePascalCompiler}
   CmdLine_Utilities,
   Misc_Utilities,
   Fakultaet;
{$ENDIF}

// ----------------------------------------------------------------------------                .
function Eval_CmdLine (var N: Integer; var be_quiet: Boolean): Boolean;
begin
   if (ParamCount() >= 1) then begin
      N := StrToIntDef(ParamStr(1),(-1));
   end else begin
      N := (-1);
   end;
   be_quiet := (ParamCount() = 2) and
               ((CmdLineSwitch('q',['-','/'],False{$IFDEF FreePascalCompiler},False{$ENDIF}) > 0) or
                (CmdLineSwitch('quiet',['-'],False,True) > 0));
   Result := (N > (-1)) and
             ((ParamCount() = 1) or (be_quiet));
end;

// ----------------------------------------------------------------------------                .
function Check_Execute (const N: Integer): Boolean;
var
   c : Char;
begin
   if (N < 10000) then begin
      Result := True;
   end else begin
      Write('This might take a while .. are you sure? ');
      Read(c);
      Result := (c in ['y','Y','j','J']);
      WriteLn;
   end;
end;

// -- Programm- & Versions-Info -------------------------------                                .
{$IFDEF DelphiPascalCompiler}
{$R 'nf.verinfo.res'}          // generiert mit VerGen.exe Copyright (c) 2002, 2005 ~MNe :-)   .
{$ENDIF}
{$IFDEF FreePascalCompiler}
{$R 'nf.verinfo.or'}           // generiert mit VerGen.exe Copyright (c) 2002, 2005 ~MNe :-)   .
{$ENDIF}
var
   Program_name : String;
   CmdLine_ok   : Integer;
// ------------------------------------------------------------                                .

const
   MAX_N     = (1 * 1024 * 1024); // Max. Fakultaet: 2^28 ..                                   .
   PROFILE_N = 15432;  // Diese Fakultaet fuer's Profiling                                     .

var
   N, i : Integer;
   be_quiet : Boolean;
   t : Cardinal;
   f : TFakultaet;
   s : String;

// ============================================================================                .

begin try
   // Vorspann des Programms                                                                   .
   Program_name := appGetApplicationFileName();
   CmdLine_ok := GetFirstInvalidSwitch(
      CheckCmdLineSwitches(  // Kommandozeile auswerten #1, "Lang-Form" ..                     .
         ['version','history',
          'quiet',
          'profile'],
         ['-'],
         [False,False,
          False,
          False],
         True),
      CheckCmdLineSwitches(  // Kommandozeile auswerten #2, "Kurz-Form" ..                     .
         ['v','h',
          'q',
          'p'],
         ['-','/'],
         [False,False,
          False,
          False]
         {$IFDEF FreePascalCompiler},False{$ENDIF}));
   WriteLn;
   WriteLn(Program_name,' -- ',resGetXVersionInfo(),' (',resGetXWersionInfo(),')',', ',
      resGetXReleaseInfo(),', ',resGetXDateTimeInfo());
   Writeln(StringOfChar(' ',Length(Program_name)),' -- ',resGetXCopyrightInfo());
   WriteLn(StringOfChar(' ',Length(Program_name)),' -- ','(dbg) OS: ',
      sysGetWin32Version().w32ComposedName,'; CmdLineCnt: ',ParamCount(),'; CmdLineErg: ',
      CmdLine_ok);
   WriteLn;
   // Kommandozeile auswerten                                                                  .
   if (CmdLine_ok = CMDLINE_VALID) and
      ((ParamCount = 1) and ((CmdLineSwitch('v',['-','/'],False{$IFDEF FreePascalCompiler},False{$ENDIF}) > 0) or
                             (CmdLineSwitch('version',['-'],False,True) > 0))) then begin
      // Parameter '--version': keine weiteren Informationen anzeigen ..                       .
      {$INCLUDE '.\all.motd.inc'}
   end else if (CmdLine_ok = CMDLINE_VALID) and
               ((ParamCount = 1) and ((CmdLineSwitch('h',['-','/'],False{$IFDEF FreePascalCompiler},False{$ENDIF}) > 0) or
                                      (CmdLineSwitch('history',['-'],False,True) > 0))) then begin
      // Parameter '-History': als Resource gelinkte History anzeigen ..                       .
      WriteLn(resGetXHistoryInfo());
   end else if (CmdLine_ok = CMDLINE_VALID) and
               ((ParamCount = 1) and ((CmdLineSwitch('p',['-','/'],False{$IFDEF FreePascalCompiler},False{$ENDIF}) > 0) or
                                      (CmdLineSwitch('profile',['-'],False,True) > 0))) then begin
      // Parameter '-p': Profiling der unterschiedlichen Methoden mit vorgegebenem N ..        .
      WriteLn('Profiling ',TFakultaet.GetBestM(),' different methodes by calculating ',
         fmtFormatNumber((1.0 * PROFILE_N),0),'!:');
      WriteLn;
      for i := 1 to TFakultaet.GetBestM() do begin
         f := TFakultaet.Create();
         Write('Calculating ',fmtFormatNumber((1.0 * PROFILE_N),0),'! /w methode #',i,' ..');
         t := GetTickCount();
         f.M := i;
         f.N := PROFILE_N;
         t := (GetTickCount() - t);
         if (f.NumDigits > 0) then begin
            // Berechnung ist OK - Ergebnis ausgeben ..                                        .
            WriteLn(' Done. Calculation needed ',fmtFormatNumber((t / 1000),2),' second(s).');
            s := Copy(f.Result,1,100);
            WriteLn('  Result snapshot: ',Copy(s,1,50));
            WriteLn('                   ',Copy(s,51,50),' ..');
         end else begin
            // Berechnung ist fehlgeschlagen - Fehlernummer und -meldung ausgeben ..           .
            WriteLn(' Done, but the following error occured:');
            WriteLn('  Error ',f.NumDigits,': ',f.Result,'.');
         end;
         FreeAndNil(f);
      end;
   end else if (CmdLine_ok = CMDLINE_VALID) and
               ((Eval_CmdLine(N,be_quiet)) and
                ((N >= 1) and (N <= MAX_N))) then begin
      if ((be_quiet) or (Check_Execute(N))) then begin
         // Prioritaet des Programmes herabsetzen, damit der Compi weiter benutzt werden kann ..  .
         if not(SetPriorityClass(GetCurrentProcess(),NORMAL_PRIORITY_CLASS)) then begin
            WriteLn('SetPriorityClass() failed: ''',fmtFormatMessageFromSystem({$IFDEF FreePascalCompiler}MAX_CARDINAL,True{$ENDIF}),
               '''; will continue anyway.');
         end;
         if not(SetThreadPriority(GetCurrentThread(),THREAD_PRIORITY_LOWEST)) then begin
            WriteLn('SetThreadPriority failed: ''',fmtFormatMessageFromSystem({$IFDEF FreePascalCompiler}MAX_CARDINAL,True{$ENDIF}),
               '''; will continue anyway.');
         end;
         // Hier geht es los mit der Berechnung von N! ..                                         .
         f := TFakultaet.Create();
         if not(be_quiet) then begin
            Write('Wait ..');
         end;
         t := GetTickCount();
      // f.M := TFakultaet.GetBestM(); << wird im Konstruktor eh eingestellt ..                   .
         f.N := N;
         t := (GetTickCount() - t);
         if (f.NumDigits > 0) then begin
            // Berechnung ist OK - Ergebnis ausgeben ..                                           .
            if not(be_quiet) then begin
               WriteLn(' Done. ',fmtFormatNumber((1.0 * f.N),0),'! has ',fmtFormatNumber(
                  (1.0 * f.NumDigits),0),' digits; calculation /w methode #',f.M,' needed ',
                  fmtFormatNumber((t / 1000),2),' second(s) and took (',fmtFormatNumber(
                  (1.0 * f.M_Iterations),0),'+',fmtFormatNumber((1.0 * f.UC_Iterations),0),
                  ') iteration(s).');
               WriteLn;
            end;
            WriteLn(fmtFormatNumber((1.0 * f.N),0),'! = ',f.Result);
         end else begin
            // Berechnung ist fehlgeschlagen - Fehlernummer und -meldung ausgeben ..              .
            WriteLn(' Done, but the following error occured:');
            WriteLn('   Error ',f.NumDigits,': ',f.Result,'.');
         end;
         FreeAndNil(f);
      end else begin
         {$INCLUDE '.\all.motd.inc'}
      end;
   end else begin
      // Ungueltigen Parameter / Schalter gefunden? -> Ausgabe desjenigen welchen ..           .
      if (CmdLine_ok <> CMDLINE_VALID) then begin
         WriteLn('Sorry! Unknown switch: ','''',ParamStr(CmdLine_ok),'''','.');
         WriteLn;
      end;
      // Angegebene Kommandozeile war wohl niX .. Usage-Meldung ausgeben ..                    .
      WriteLn('Usage: ',Program_name,' [N] {-q|--quiet} | {-v|--version} | {-h|' +
         '--history} | {-p|--profile}');
      WriteLn;
      WriteLn(' e.g.: ',Program_name,' 666');
      WriteLn;
      WriteLn('  N                  A number between 1 (min) and ',
         fmtFormatNumber((1.0 * MAX_N),0),' (max).');
      WriteLn('  -q, /q, --quiet    Quiet: no additional output.');
      WriteLn('  -v, /v, --version  Display version number only.');
      WriteLn('  -h, /h, --history  Display program/ version history.');
      WriteLn('  -p, /p, --profile  Profile the ',TFakultaet.GetBestM(),' diffe' +
         'rent implementations.');
      WriteLn;
      WriteLn('Hint(s): Parameters enclosed in ''[]'' are mandatory; parameters' +
         ' enclosed in ''{}'' are optional; parameters seperated by ''|'' exclu' +
         'de each other; ''<..>'' within a parameter means that *you* have to s' +
         'upply additional information.');
      WriteLn;
      WriteLn('Beware: While calculating 999! (2.565 digits: ''40238726007..'')' +
         ' only takes about 00:00,20 [mm:ss,hh], calculating 99.999! (456.569 d' +
         'igits: ''28242294079..'') takes about 59:25,50 [mm:ss,hh]! (System wa' +
         's a P3 @ 550 MHz /w 512 MB, Windows NT 4.0 /w SP6a).');
      WriteLn;
      WriteLn('Bug report: Pls. report all bug(s) and/ or wishes, suggestions e' +
         'tc. to <BugReport@nefzger.at>, including the module name (which is: ''',
         Program_name,''') and module version (which is: ''',resGetXVersionInfo(),
         ''' [''',resGetXWersionInfo(),''']','). Thank you.');
   end;
   Halt(0); // Normales Programmende, mit Exitcode 0 retour ..                                 .
except
   on e: Exception do begin
      {$INCLUDE '.\all.fatalerror.inc'}
      WriteLn(Error,' Fatal: Exception: ','''',e.Message,'''','; pls. contact t' +
         'he programmer: <BugReport@nefzger.at>.');
      Halt(1); // Abnormales Programmende: irgendwo wurde (intern) eine Exception geworfen ..  .
   end;
end; {try} end. // {begin}                                                                     .

